/*
  Case FAN Control Driver for GLANTANK
  Copyright (c) 2006 ZOBplus hiro <hiro@zob.ne.jp>

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of th License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/proc_fs.h>
#include <linux/stringify.h>
#include <linux/seq_file.h>
#include <asm/uaccess.h>

#define FUNDRV_MAJOR 254
#define VERSION "1.00"

#define BOARD_ID_G    0
#define BOARD_ID_GWGZ 1

#define CPLD_BASE               0xfe8d0000
#define CPLD_FAN_CTRL           (*(volatile u8 *)(CPLD_BASE + 6))

static struct proc_dir_entry *fan_proc_entry;
static int mask = 1;
module_param(mask, int, 0644);

static int
fandrv_read_state(struct seq_file *seq, void *offset)
{
  seq_printf(seq, "status: %02x\n", CPLD_FAN_CTRL);
  return 0;
}

static int
fandrv_proc_open(struct inode *inode, struct file *file)
{
  return single_open(file, fandrv_read_state, PDE(inode)->data);
}

static int
fandrv_proc_write(struct file *file, const char __user *buffer,
		  size_t count, loff_t *ppos)
{
  unsigned long v;
  char lbuffer[16];
  if (count > sizeof(lbuffer) - 1) {
    return -EINVAL;
  }
  memset(lbuffer, 0, sizeof(lbuffer));
  if (copy_from_user(lbuffer, buffer, count)) {
    return -EFAULT;
  }
  v = simple_strtol(lbuffer, 0, 0);
  if (mask) {
    v = (v == 0) ? 0 : 1;
  }

  CPLD_FAN_CTRL = v;
  return count;
}

struct file_operations fandrv_proc_fops = {
  .owner    = THIS_MODULE,
  .open     = fandrv_proc_open,
  .llseek   = seq_lseek,
  .read     = seq_read,
  .write    = fandrv_proc_write,
  .release  = single_release,
};

static int __init
fandrv_init_module(void)
{
  if ((fan_proc_entry = create_proc_entry("fandrv", 0, 0)) == NULL) {
    printk(KERN_ERR "fail to create /proc/fandrv.");
    return -EBUSY;
  }
  fan_proc_entry->proc_fops = &fandrv_proc_fops;
  printk("FAN control driver initialized.\n");
  return 0;
}

static void __exit
fandrv_cleanup_module(void)
{
  remove_proc_entry("fandrv", NULL);
  printk("FAN control driver cleanup.\n");
}

module_init(fandrv_init_module);
module_exit(fandrv_cleanup_module);

MODULE_AUTHOR("ZOBplus hiro <hiro@zob.ne.jp>");
MODULE_DESCRIPTION("GLANTANK Case Fan Control Driver");
MODULE_LICENSE("GPL");
MODULE_VERSION(VERSION);
